#include "CppUnitTest.h"
#include "CalculatorPresentationModel.h"

using namespace Microsoft::VisualStudio::CppUnitTestFramework;
using namespace Calculator::View;

namespace Calculator
{
	namespace Testing
	{
		TEST_CLASS(PresentationModelTest)
		{
		public:
			TEST_METHOD_INITIALIZE(TestInit)
			{
				presentationModel_.ClickedClear();
			}

			TEST_METHOD(TestAdd)
			{
				// 784 + 324 = 1108
				presentationModel_.ClickedNumber(7);
				presentationModel_.ClickedNumber(8);
				presentationModel_.ClickedNumber(4);

				presentationModel_.ClickedOperator(OpAdd);

				presentationModel_.ClickedNumber(3);
				presentationModel_.ClickedNumber(2);
				presentationModel_.ClickedNumber(4);

				presentationModel_.ClickedOperator(OpNone);

				Assert::AreEqual<long>(1108, presentationModel_.GetAccum(), L"Add operation leads to wrong result.");
				Assert::AreEqual<CalcError>(ErrNone, presentationModel_.GetErrorState(), L"Add operation ends with wrong error state.");
			}

			TEST_METHOD(TestSubtract)
			{
				// 784 - 324 = 460
				presentationModel_.ClickedNumber(7);
				presentationModel_.ClickedNumber(8);
				presentationModel_.ClickedNumber(4);

				presentationModel_.ClickedOperator(OpSubtract);

				presentationModel_.ClickedNumber(3);
				presentationModel_.ClickedNumber(2);
				presentationModel_.ClickedNumber(4);

				presentationModel_.ClickedOperator(OpNone);

				Assert::AreEqual<long>(460, presentationModel_.GetAccum(), L"Subtract operation leads to wrong result.");
				Assert::AreEqual<CalcError>(ErrNone, presentationModel_.GetErrorState(), L"Subtract operation ends with wrong error state.");
			}

			TEST_METHOD(TestMultiply)
			{
				// 784 * 324 = 254016
				presentationModel_.ClickedNumber(7);
				presentationModel_.ClickedNumber(8);
				presentationModel_.ClickedNumber(4);

				presentationModel_.ClickedOperator(OpMultiply);

				presentationModel_.ClickedNumber(3);
				presentationModel_.ClickedNumber(2);
				presentationModel_.ClickedNumber(4);

				presentationModel_.ClickedOperator(OpNone);

				Assert::AreEqual<long>(254016, presentationModel_.GetAccum(), L"Multiply operation leads to wrong result.");
				Assert::AreEqual<CalcError>(ErrNone, presentationModel_.GetErrorState(), L"Multiply operation ends with wrong error state.");
			}

			TEST_METHOD(TestDivide)
			{
				// 784 / 324 = 2 (integer division)
				presentationModel_.ClickedNumber(7);
				presentationModel_.ClickedNumber(8);
				presentationModel_.ClickedNumber(4);

				presentationModel_.ClickedOperator(OpDivide);

				presentationModel_.ClickedNumber(3);
				presentationModel_.ClickedNumber(2);
				presentationModel_.ClickedNumber(4);

				presentationModel_.ClickedOperator(OpNone);

				Assert::AreEqual<long>(2, presentationModel_.GetAccum(), L"Divide operation leads to wrong result.");
				Assert::AreEqual<CalcError>(ErrNone, presentationModel_.GetErrorState(), L"Divide operation ends with wrong error state.");
			}

			TEST_METHOD(TestDivideByZero)
			{
				// 784 / 0 => ErrDivideByZero
				presentationModel_.ClickedNumber(7);
				presentationModel_.ClickedNumber(8);
				presentationModel_.ClickedNumber(4);

				presentationModel_.ClickedOperator(OpDivide);

				presentationModel_.ClickedNumber(0);

				presentationModel_.ClickedOperator(OpNone);

				Assert::AreEqual<CalcError>(ErrDivideByZero, presentationModel_.GetErrorState(), L"Divide by zero doesnt end with error state.");
			}
		private:
			CalculatorPresentationModel presentationModel_;
		};
	}
}