#include "CalculatorPresentationModel.h"

using namespace Calculator::View;

CalculatorPresentationModel::CalculatorPresentationModel(ICalculatorView* view) : 
	view_(view) {}

void CalculatorPresentationModel::PerformOperation()
{
	if (m_errorState != ErrNone)
		return;

	if (m_bOperandAvail) {
		if (m_operator == OpNone)
			m_accum = m_operand;
		else if (m_operator == OpMultiply)
			m_accum *= m_operand;
		else if (m_operator == OpDivide) {
			if (m_operand == 0)
				m_errorState = ErrDivideByZero;
			else
				m_accum /= m_operand;
		}
		else if (m_operator == OpAdd)
			m_accum += m_operand;
		else if (m_operator == OpSubtract)
			m_accum -= m_operand;
	}

	m_bOperandAvail = false;
	UpdateDisplay();
}

void CalculatorPresentationModel::ClickedNumber(long l)
{
	if (m_errorState != ErrNone)
		return;

	if (!m_bOperandAvail)
		m_operand = 0L;

	SetOperand(m_operand*10+l);
	UpdateDisplay();
}

void CalculatorPresentationModel::ClickedClear(void)
{
	m_operator = OpNone;
	m_operand = 0L;
	m_accum = 0L;
	m_bOperandAvail = false;
	m_errorState = ErrNone;
	UpdateDisplay();
}

void CalculatorPresentationModel::ClickedOperator(Operator oper)
{
	PerformOperation();
	m_operator = oper;
}

inline void CalculatorPresentationModel::UpdateDisplay(void)
{
	if (view_)
		view_->UpdateDisplay();
}

long CalculatorPresentationModel::GetAccum()
{
	return m_accum;
}

void CalculatorPresentationModel::SetAccum(long nNewValue)
{
	m_accum = nNewValue;
}

long CalculatorPresentationModel::GetOperand()
{
	return m_operand;
}

void CalculatorPresentationModel::SetOperand(long nNewValue)
{
	m_operand = nNewValue;
	m_bOperandAvail = true;
}

long CalculatorPresentationModel::GetValue()
{
	return m_bOperandAvail ? m_operand : m_accum;
}

short CalculatorPresentationModel::GetOperation()
{
#pragma warning(disable:4244)
	return m_operator;
#pragma warning(default:4244)
}

void CalculatorPresentationModel::SetOperation(short nNewValue)
{
	m_operator = (Operator)nNewValue;
}

CalcError CalculatorPresentationModel::GetErrorState()
{
	return m_errorState;
}