#ifndef CALCULATOR_PRESENTATION_MODEL_H_
#define CALCULATOR_PRESENTATION_MODEL_H_

#include <memory>

using namespace std;

namespace Calculator
{
	namespace View
	{
		enum Operator { OpNone, OpAdd, OpSubtract, OpMultiply, OpDivide };
		enum CalcError { ErrNone, ErrDivideByZero };

		class ICalculatorView
		{
		public:
			virtual void UpdateDisplay(void) = 0;
		};

		class CalculatorPresentationModel
		{
		public:
			CalculatorPresentationModel(ICalculatorView* = nullptr);
			void ClickedNumber(long lNum);
			void ClickedClear(void);
			void ClickedOperator(Operator);
			void UpdateDisplay();

			long GetAccum();
			void SetAccum(long nNewValue);
			long GetOperand();
			void SetOperand(long nNewValue);
			long GetValue();
			short GetOperation();
			void SetOperation(short nNewValue);
			CalcError GetErrorState();

		private:
			// PresentationModel partner
			ICalculatorView* view_;

			// PresentationModel state
			long m_accum;
			long m_operand;
			Operator m_operator;
			CalcError m_errorState;
			bool m_bOperandAvail;

			void PerformOperation();
		};
	}
}

#endif