﻿//
// MainPage.xaml.cpp
// Implementation of the MainPage class.
//

#include "pch.h"
#include "MainPage.xaml.h"

using namespace XamlCalc;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::System;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

MainPage::MainPage() :
	view_(CalcView(this)),
	presentationModel_(new CalculatorPresentationModel(&view_))
{
	InitializeComponent();
}

/// <summary>
/// Invoked when this page is about to be displayed in a Frame.
/// </summary>
/// <param name="e">Event data that describes how this page was reached.  The Parameter
/// property is typically used to configure the page.</param>
void MainPage::OnNavigatedTo(NavigationEventArgs^ e)
{
	(void) e;	// Unused parameter
}

void MainPage::Page_Loaded(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	// clear the contents of the calculator and reset state
	presentationModel_->ClickedClear();
}

void MainPage::Key_Press(Platform::Object^ sender, Windows::UI::Core::KeyEventArgs^ e)
{
	VirtualKey key_pressed = e->VirtualKey;

	switch (key_pressed)
	{
	case VirtualKey::C:
		presentationModel_->ClickedClear();
		break;
	case VirtualKey::Divide:
		presentationModel_->ClickedOperator(OpDivide);
		break;
	case VirtualKey::Add:
		presentationModel_->ClickedOperator(OpAdd);
		break;
	case VirtualKey::Subtract:
		presentationModel_->ClickedOperator(OpSubtract);
		break;
	case VirtualKey::Multiply:
		presentationModel_->ClickedOperator(OpMultiply);
		break;
	case static_cast<VirtualKey>(187):
		presentationModel_->ClickedOperator(OpNone);
		break;
	default:
		if (key_pressed>=VirtualKey::NumberPad0 && key_pressed<=VirtualKey::NumberPad9)
			presentationModel_->ClickedNumber(safe_cast<long>(key_pressed - VirtualKey::NumberPad0));
		break;
	}
}

void MainPage::Number_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	Button^ b = safe_cast<Button^>(sender);
	long nID = (safe_cast<String^>(b->Content)->Data())[0] - L'0';
	presentationModel_->ClickedNumber(nID);
}

void MainPage::Clear_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	presentationModel_->ClickedClear();
}

void MainPage::Divide_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	presentationModel_->ClickedOperator(OpDivide);
}

void MainPage::Equal_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	presentationModel_->ClickedOperator(OpNone);
}

void MainPage::Minus_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	presentationModel_->ClickedOperator(OpSubtract);
}

void MainPage::Plus_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	presentationModel_->ClickedOperator(OpAdd);
}

void MainPage::Multiply_Click(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{
	presentationModel_->ClickedOperator(OpMultiply);
}

void MainPage::UpdateDisplay()
{
	display_->Text = (presentationModel_->GetErrorState() != ErrNone) ?
		L"ERROR!" :
		safe_cast<int64_t>(presentationModel_->GetValue()).ToString();

	button_equal_->Focus(Windows::UI::Xaml::FocusState::Programmatic);
}

CalcView::CalcView(MainPage^ page) : page_(page) {}

inline void CalcView::UpdateDisplay()
{
	page_->UpdateDisplay();
}